function demo_effect_of_boundary_handling_on_signal
% illustration: how many coefs and signal samples is affected by boundary
% handling

clc
close all

N = 85;
% signal is an excerpt from image 'tire', crop: tire(120,65:end)
signal = [81,94,96,110,135,150,155,147,153,156,147,148,183,236,250,240,219,203,204,202,197,193,187,175,159,141,124,107,97,88,73,66,60,52,49,56,65,74,64,62,76,107,160,186,172,163,176,191,176,122,85,54,45,49,41,39,47,53,60,76,88,98,115,134,147,154,157,157,149,154,165,198,247,255,247,193,140,103,42,29,65,97,102,106,104,131,132,114,99,86,68,53,28,10,18,15,11,16,19,24,31,38,50,66,78,109,130,145,154,154,122,70,31,15,16,27,32,38,28,8,3,10,11,10,13,12,13,10,12,15,12,11,13,13,14,13,13,15,15,13,14,15,14,15,13,13,11,11,11,10,20,54,53,38,46,49,155,251,252,254,255,255,255,255,255,254,255,185];


%% main parameters
wavelet = 'db6'
[Lo_D, Hi_D] = wfilters(wavelet,'d');
fillen = length(Lo_D);

type = {'zpd'; 'sym'; 'asym'; 'sp1'};
style = {'kx-' 'ko--' 'k*-.' 'kv:'};
% style = {'r' 'g' 'k' 'y'};
% grafu = 4;

emph_factor = 5; % for emphasizing the result (i.e. differences) by multiplication

no_aff_coef = n_affect_l(1,fillen); %j=1
no_samples = no_aff_coef * 2^1 %j=1


%% Overall graph - first part
f_overall = figure;
h_overall = plot(signal(1:25),'k');
hold on
h1_overall = plot(signal(1:no_samples),'k','LineWidth',2.5);
xlabel('signal samples')
title('Original signal and its reconstructions from the modified coefficients')


%% Zoomed graph - first part
f_detail = figure;
h_detail = plot(signal(1:no_samples),'k','LineWidth',2);
hold on
xlabel('signal samples')
title('Detail: Original signal and its reconstructions from the modified coefficients')

%% Coefficients graphs
f_coefs = figure;

for cnt = 1:length(type)
    
    %forward DWT
    dwtmode(type{cnt});
    [C,L] = wavedec(signal,1,wavelet);
    
    %plotting the detail coeffecients
    figure(f_coefs)
    subplot(2,1,1)
    plot(C((L(1)+1):(L(1)+no_aff_coef)),style{cnt})
%     stem(C((L(1)+1):(L(1)+no_aff_coef)),style{cnt})
    title('Left-side detail coefficients influenced by the values of boundary extensions')
    xlabel('coefficient index')
    axis tight
    hold on
    %plotting the approx. coeffecients
    subplot(2,1,2)
    plot(C(1:no_aff_coef),style{cnt})
%     stem(C(1:no_aff_coef),style{cnt})
    title('Left-side approximation coefficients influenced by the values of boundary extensions')
    xlabel('coefficient index')
    axis tight
    hold on

    %modification of coefs
    C((L(1)+1):(L(1)+no_aff_coef)) = modify_coefs(C((L(1)+1):(L(1)+no_aff_coef)),emph_factor);
    
    %inverse IDWT
    signal_rec = waverec(C,L,wavelet);
    
    %back to overall figure
    figure(f_overall)
    handl_overall(cnt) = plot(signal_rec(1:no_samples),style{cnt});
    
    %back to detail figure
    figure(f_detail)
    handl_detail(cnt) = plot(signal_rec(1:no_samples),style{cnt});
    axis tight
    
end

legend([h_overall handl_overall], ['original'; type])
hold off



%% inner function
function Cupr = modify_coefs(C,multiplier)
%nonlinear (thresholding)
lambda = 9;
Cupr = C;
Cupr(abs(C)<lambda) = 0;
Cupr = multiplier * Cupr; %emphasizement
return