function [C, L] = mldwt(signal, J, Lo_D, Hi_D, sampling_parity)

% MLDWT - Multi-level discrete-time Wavelet Transfrom
% Basic demostrative implementation of DWT for an arbitrary depth.
% Boundary extension type is zero padding.
%
% signal.............column vector of input signal
% J..................depth of the transform
% Lo_D, Hi_D.........wavelet filters
% sampling_parity....even (0, default) or odd (1) downsampling; see help to DOWNSAMPLE
%
% L..................lenghts of vectors of coefficients
%         L(J+2) - length of the input signal
% C..................coefficients by columns
%         C(1:L(i), i) - detail coefs, i = 1,...,J  
%         C(1:L(J), J) - approximation coefs

%(c) 2010-12 Robert Konczi, Pavel Rajmic, Brno University of Technology, Czech Republic


%% input check etc.
signal = signal(:);
if nargin == 4
    sampling_parity = 0; % default
end

%% computing
L(J+2) = length(signal);

% loop
for j = 1:J
    % single-level decomposition
    [Lo_coef, Hi_coef] = sldwt(signal, Lo_D, Hi_D, sampling_parity);
    % keep the approx. coefs
    signal = Lo_coef;
    % output detail coefs
    C(1:length(Hi_coef),j) = Hi_coef;
    % output the respective lenght
    L(j) = length(Hi_coef');
end

% approximation coefs at the last level
C(1:length(signal),J+1) = signal;
L(J+1) = length(signal);