function signal = mlidwt(C, L, Lo_R, Hi_R, sampling_parity)

% MLDWT - Multi-level inverse discrete-time Wavelet Transfrom
% Basic demostrative implementation of IDWT for an arbitrary depth.
%
% L..................lenghts of vectors of coefficients (see MLDWT)
% C..................wavelet coefficients (see MLDWT)
% Lo_R, Hi_R.........reconstruction wavelet filters
% sampling_parity....even (0, default) or odd (1) downsampling; see help to DOWNSAMPLE
%
% signal.............column vector of the reconstructed signal

%(c) 2010-12 Robert Konczi, Pavel Rajmic, Brno University of Technology, Czech Republic


%% input check etc.
if nargin == 4
    sampling_parity = 0; % default
end
J = length(L)-2;

%% initialize
approx_temp = C(1:L(J+1), J+1);

%% loop
for j = J:-1:1
    
    %signal parity
    if j>1
        signal_parity = parity(L(j-1));
    else
        signal_parity = parity(L(J+2));
    end
    
    %single-level inverse wavelet transform 
    approx_temp = slidwt(approx_temp, C(1:L(j),j), Lo_R, Hi_R, signal_parity, sampling_parity);
end

% transpose
signal = approx_temp';