function [coef_Lo, coef_Hi] = sldwt(signal, Lo_D, Hi_D, sampling_parity)

% DWT - Discrete-time Wavelet Transfrom
% Basic demostrative implementation of DWT for depth J=1.
% Boundary extension type is zero padding.
%
% signal.............column vector of input signal
% Lo_D, Hi_D.........wavelet filters
% sampling_parity....even (0, default) or odd (1) downsampling; see help to DOWNSAMPLE
%
% coef_Lo, coef_Hi...resultant two vector of coefficients

%(c) 2010-12 Pavel Rajmic, Brno University of Technology, Czech Republic


%% check of parameters
if nargin == 3
    sampling_parity = 0; % default
end

n_Lo = length(Lo_D);
n_Hi = length(Hi_D);
%Remark: If lenghts of 'n_Lo' a 'n_Hi' would not be identical,
% the function computes the transform even so, but the lengths of the
% resultant (detail and approximation coefs) can differ!

%% extending
signal_ext = [zeros(n_Lo-1,1); signal; zeros(n_Lo-1,1)];

%% filtering
coef_Lo = conv(signal_ext,Lo_D);
coef_Hi = conv(signal_ext,Hi_D);

%% cutting off
coef_Lo = coef_Lo((n_Lo):(end-n_Lo+1));
coef_Hi = coef_Hi((n_Hi):(end-n_Hi+1));

%% downsampling
coef_Lo = downsample(coef_Lo,sampling_parity);
coef_Hi = downsample(coef_Hi,sampling_parity);