function signal_rec = slidwt(coef_Lo, coef_Hi, Lo_R, Hi_R, signal_parity, sampling_parity)

% iDWT - Inverse Discrete-time Wavelet Transfrom
% Basic demostrative implementationof iDWT for depth J=1.
%
% coef_Lo, coef_Hi...wavelet coefficients
% Lo_R, Hi_R.........reconstructing filters
% signal_parity......parity of the length of the original signal which is
%                    to be reconstructed
%                    it is indeed sufficient to provide just its parity,
%                    because the number itself can be traced back for it
%                    (0...even, 1...odd)
% sampling_parity....even (0, default) or odd (1) upsampling;
%                    see also help for UPSAMPLE
%
% signal_rec.........reconstructed signal

%(c) 2010-12 Pavel Rajmic, Brno University of Technology, Czech Republic


%% parameters check
if nargin == 5
    sampling_parity = 0; % default
end

%% upsampling the coefficients vectors
coef_Lo_up = upsample(coef_Lo,sampling_parity);
coef_Hi_up = upsample(coef_Hi,sampling_parity);

%% filtering
signal_Lo = conv(coef_Lo_up,Lo_R);
signal_Hi = conv(coef_Hi_up,Hi_R);

%% sum
signal_sum = signal_Lo + signal_Hi;
% length(signal_sum)      %length check - real
% 2*coef_len + filter_len %length check - theoretic

%% cutting off
% computation of the original length out from its parity
filter_len = length(Lo_R);  %m
coef_len = length(coef_Lo); %p
filter_parity = parity(filter_len);

if parity(signal_parity+filter_parity-1) == 0 %if this term is even
    sig_len = 2*coef_len - filter_len + 1;% - 2*sampling_parity;
else %lichy
    sig_len = 2*coef_len - filter_len - 2*sampling_parity + 2;
end

%indexes to retain:
%the starting index of the retained samples
start = filter_len - filter_parity - 1 + 2*sampling_parity; 
%crop
signal_rec = signal_sum( start : (start+sig_len-1) );