% This program generates Matlab fuction "w_filters.m", which can be used to
% get the impulse responses of wavelet filters
%
% The resultant "w_filters.m" is thus independent of the official wavelet
% toolbox, where the corresponding function is "wfilters".

% It might print error messages for some wavelets, but everything is ok :)

% (c) 2012, Z. Prusa, P. Rajmic, Brno University of Technology

functionname = 'w_filters';
filename = [functionname, '.m'];
wnames =  wavenames('all');

%% Write the headline
% dlmwrite(filename,' '); %rewrites the original file, at the same time
dlmwrite(filename, ['function [Lo_D, Hi_D, Lo_R, Hi_R] = ', functionname, '(wname)'], 'delimiter', '');
dlmwrite(filename, [' '], '-append', 'delimiter', '');

%% Store the names
for i=1:length(wnames)
    names{i} = [wnames{i} ', ' ];
end

%% writing help
dlmwrite(filename, ['% W_FILTERS returns four impulse responses of filters under the abbrev. ''wname'' '], '-append', 'delimiter', '');
dlmwrite(filename, ['% Options for ''wname'': '], '-append', 'delimiter', '');
dlmwrite(filename, ['% ' names{2:end}], '-append', 'delimiter', '');
dlmwrite(filename, [' '], '-append', 'delimiter','');
%% copyright
dlmwrite(filename, ['% (c) 2010-12 Zdenek Prusa, Pavel Rajmic, Brno University of Technology, Czech Republic'], '-append', 'delimiter', '');
dlmwrite(filename, [' '], '-append', 'delimiter','');
%% date and version
dlmwrite(filename, ['% Generated by function ''w_filters_generation'' on ', datestr(date)], '-append', 'delimiter', '')
struct = ver('wavelet');
dlmwrite(filename, ['% using Matlab Wavelet Toolbox version ', struct.Version], '-append', 'delimiter', '')
dlmwrite(filename, [' '], '-append', 'delimiter','');
dlmwrite(filename, [' '], '-append', 'delimiter','');



%% code itself
dlmwrite(filename, ['switch wname'], '-append', 'delimiter', '');
wfiltersFailed = 0;
for i = 2:length(wnames)
    
    % Identifying wavelets that are defined by the lifting scheme
    try
     [LO_A, HI_A, LO_R, HI_R] = wfilters(wnames{i});
    catch err
        LS = liftwave(wnames{i});
        [LO_A, HI_A, LO_R, HI_R] = ls2filt(LS);
    end

    % Removing possible zeros from the beginning
    if LO_A(1) == 0 && HI_A(1) == 0
        LO_A = LO_A(2:end);HI_A = HI_A(2:end);
    end
    
    if LO_R(1) == 0 &&  HI_R(1) == 0
        LO_R = LO_R(2:end);HI_R = HI_R(2:end);
    end
    
    % Write the impulse responses
    dlmwrite(filename, ['%', num2str(i-1)], '-append', 'delimiter', '');
    dlmwrite(filename, ['case ''' wnames{i} ''''], '-append', 'delimiter', '');
    dlmwrite(filename, ['Lo_D = [ ' num2str(LO_A,15) ' ];'], '-append', 'delimiter', '');
    dlmwrite(filename, ['Hi_D = [ ' num2str(HI_A,15) ' ];'], '-append', 'delimiter', '');
    dlmwrite(filename, ['Lo_R = [ ' num2str(LO_R,15) ' ];'], '-append', 'delimiter', '');
    dlmwrite(filename, ['Hi_R = [ ' num2str(HI_R,15) ' ];'], '-append', 'delimiter', '');
    dlmwrite(filename, ' ', '-append', 'delimiter', '');
    
end

dlmwrite(filename, ['otherwise error(''Not supported wname'')'], '-append', 'delimiter', '');
dlmwrite(filename, ['end'],'-append','delimiter','');